#!/bin/bash

#set -x
echo "Logging CAN stuff"

_now=$(date +%Y_%m_%d_%H%M%S)        



TIMEOUT=2
MESSAGE_CNT=1
MATCHSTRING="messages sent and received: $MESSAGE_CNT"

case "$1" in
	can_fdtest)
		READER=`perl -e "alarm $TIMEOUT; exec @ARGV" "canfdtest $2"`
		;;
		
    can_loopback)
		if [ -z "$2" ]; then
		   CAN_PORT="can0"
		else
		   CAN_PORT=$2
		fi

		if [ -z "$3" ]; then
		   BAUD=250000
		else
		   BAUD=$3
		fi

		if [ -z "$4" ]; then
		   RESET=""
		else
		   RESET=$4
		fi

		if [ -z "$5" ]; then
		   DEBUG=""
		else
		   DEBUG=$5
		fi
		COUNT_FILE=/var/volatile/${CAN_PORT}.count
		FAILCOUNT_FILE=/var/volatile/${CAN_PORT}.failcount
		PASSCOUNT_FILE=/var/volatile/${CAN_PORT}.passcount
		
		if [ "$4" = "reset" ]; then
			echo 0 > $COUNT_FILE
			echo 0 > $FAILCOUNT_FILE
			echo 0 > $PASSCOUNT_FILE
		fi
				
		ip link set $CAN_PORT up type can bitrate $BAUD restart-ms 100
		canfdtest $CAN_PORT & #listen for CAN packets?
        while [ 1 ]; do
			if [ -f $COUNT_FILE ]; then
				COUNT=`cat $COUNT_FILE`
			fi
			if [ -f $FAILCOUNT_FILE ]; then
				FAILCOUNT=`cat $FAILCOUNT_FILE`
			fi
			if [ -f $PASSCOUNT_FILE ]; then
				PASSCOUNT=`cat $PASSCOUNT_FILE`
			fi
			
			RESULT=`perl -e "alarm $TIMEOUT; exec @ARGV" "canfdtest -g -l $MESSAGE_CNT $CAN_PORT"`
			RESULT_TRIM=`echo $RESULT | grep "sent and received:"`
			RESULT2=`echo $RESULT_TRIM | grep "$MATCHSTRING"`
			COUNT=`expr $COUNT + 1`
			if [ "$RESULT2" ]; then  
				PASSCOUNT=`expr $PASSCOUNT + 1`
			else
				FAILCOUNT=`expr $FAILCOUNT + 1`
			fi
			echo $COUNT > $COUNT_FILE
			echo $FAILCOUNT > $FAILCOUNT_FILE
			echo $PASSCOUNT > $PASSCOUNT_FILE
		done
        ;;
	can_bridge)
		if [ -z "$2" ]; then
		   CAN_PORT_A="can0"
		else
		   CAN_PORT_A=$2
		fi

		if [ -z "$3" ]; then
		   CAN_PORT_B="can1"
		else
		   CAN_PORT_B=$3
		fi
		
		if [ -z "$4" ]; then
		   BAUD=250000
		else
		   BAUD=$4
		fi

		if [ -z "$5" ]; then
		   RESET=""
		else
		   RESET=$5
		fi

		if [ -z "$6" ]; then
		   DEBUG=""
		else
		   DEBUG=$6
		fi
		COUNT_FILE_A=/var/volatile/${CAN_PORT_A}.count
		FAILCOUNT_FILE_A=/var/volatile/${CAN_PORT_A}.failcount
		PASSCOUNT_FILE_A=/var/volatile/${CAN_PORT_A}.passcount
		
		COUNT_FILE_B=/var/volatile/${CAN_PORT_B}.count
		FAILCOUNT_FILE_B=/var/volatile/${CAN_PORT_B}.failcount
		PASSCOUNT_FILE_B=/var/volatile/${CAN_PORT_B}.passcount
		
		
		if [ $RESET = "reset" ]; then
			echo 0 > $COUNT_FILE_A
			echo 0 > $FAILCOUNT_FILE_A
			echo 0 > $PASSCOUNT_FILE_A
			echo 0 > $COUNT_FILE_B
			echo 0 > $FAILCOUNT_FILE_B
			echo 0 > $PASSCOUNT_FILE_B
		fi
				
		ip link set $CAN_PORT_A up type can bitrate $BAUD restart-ms 100
		ip link set $CAN_PORT_B up type can bitrate $BAUD restart-ms 100
		
#		canfdtest $CAN_PORT_A & #listen for CAN packets
#		canfdtest $CAN_PORT_B & #listen for CAN packets
		
        while [ 1 ]; do
			#First Test Port A
			if [ -f $COUNT_FILE_A ]; then
				COUNT_A=`cat $COUNT_FILE_A`
			fi
			if [ -f $FAILCOUNT_FILE_A ]; then
				FAILCOUNT_A=`cat $FAILCOUNT_FILE_A`
				echo "Failure $COUNT_A: $RESULT2 " > /var/volatile/can.FAILLOGA
			fi
			if [ -f $PASSCOUNT_FILE_A ]; then
				PASSCOUNT_A=`cat $PASSCOUNT_FILE_A`
			fi
			
			/usr/local/src/selftest/CANlooptest.sh can_fdtest $CAN_PORT_B &
			
			sleep 1
			
			RESULT=`perl -e "alarm $TIMEOUT; exec @ARGV" "canfdtest -g -l $MESSAGE_CNT $CAN_PORT_A"`
			RESULT_TRIM=`echo $RESULT | grep "sent and received:"`
			RESULT2=`echo $RESULT_TRIM | grep "$MATCHSTRING"`
			COUNT_A=`expr $COUNT_A + 1`
			if [ "$RESULT2" ]; then  
				PASSCOUNT_A=`expr $PASSCOUNT_A + 1`
			else
				FAILCOUNT_A=`expr $FAILCOUNT_A + 1`
				echo "Failure $COUNT_A: $RESULT" >> /var/volatile/$CAN_PORT_A.debuglog
			fi
			echo $COUNT_A > $COUNT_FILE_A
			echo $FAILCOUNT_A > $FAILCOUNT_FILE_A
			echo $PASSCOUNT_A > $PASSCOUNT_FILE_A
			if [ "$DEBUG" ]; then 
				echo "Port: $CAN_PORT_A Cnt: $COUNT_A Pass: $PASSCOUNT_A Fail: $FAILCOUNT_A"
				echo "Result: $RESULT"
			fi
				
			sleep $TIMEOUT
			
			#Then Test Port B
			if [ -f $COUNT_FILE_B ]; then
				COUNT_B=`cat $COUNT_FILE_B`
			fi
			if [ -f $FAILCOUNT_FILE_B ]; then
				FAILCOUNT_B=`cat $FAILCOUNT_FILE_B`
			fi
			if [ -f $PASSCOUNT_FILE_B ]; then
				PASSCOUNT_B=`cat $PASSCOUNT_FILE_B`
			fi
			
			/usr/local/src/selftest/CANlooptest.sh can_fdtest $CAN_PORT_A &
			sleep 1
			
			RESULT=`perl -e "alarm $TIMEOUT; exec @ARGV" "canfdtest -g -l $MESSAGE_CNT $CAN_PORT_B"`
			RESULT_TRIM=`echo $RESULT | grep "sent and received:"`
			RESULT2=`echo $RESULT_TRIM | grep "$MATCHSTRING"`
			COUNT_B=`expr $COUNT_B + 1`
			if [ "$RESULT2" ]; then  
				PASSCOUNT_B=`expr $PASSCOUNT_B + 1`
			else
				FAILCOUNT_B=`expr $FAILCOUNT_B + 1`
				echo "Failure $COUNT_B: $RESULT" >> /var/volatile/$CAN_PORT_B.debuglog
			fi
			echo $COUNT_B > $COUNT_FILE_B
			echo $FAILCOUNT_B > $FAILCOUNT_FILE_B
			echo $PASSCOUNT_B > $PASSCOUNT_FILE_B
			if [ "$DEBUG" ]; then 
				echo "Port: $CAN_PORT_B Cnt: $COUNT_B Pass: $PASSCOUNT_B Fail: $FAILCOUNT_B"
				echo "Result: $RESULT"
			fi
			sleep $TIMEOUT
		done
		;;

esac



