#!/bin/bash

#set -x
#echo "Micro Interface Test"

_now=$(date +%Y_%m_%d_%H%M%S)        

BAUD="115200"
SERIAL_COMMAND="CSV\r\n"
SERIAL_RESPONSE="CSV"
TIMEOUT=2
SCRIPTNAME=micro_monitor_collection.sh

serial_response() {

	if [ ! -c $1 ]; then
		echo "$1 Doesn't Exist"
		return 1
	fi
	
	COUNT=`cat $2`
	PASSCOUNT=`cat $3`
	FAILCOUNT=`cat $4`
	
	RESULT=`perl -e "alarm $TIMEOUT; exec @ARGV" "cat $1"`
	RESULT2=`echo $RESULT | grep "$SERIAL_RESPONSE"`
	COUNT=`expr $COUNT + 1`
	if [ "$RESULT2" ]; then  
		PASSCOUNT=`expr $PASSCOUNT + 1`
	else
		FAILCOUNT=`expr $FAILCOUNT + 1`
	fi
	echo $COUNT > $2
	echo $PASSCOUNT > $3
	echo $FAILCOUNT > $4
	
	RESULT3=`echo $RESULT | grep "Camera"`
	if [ "$RESULT3" ]; then  
		echo "Error Reading Interface"
	else
		echo $RESULT >> $5 			#LOGFILE
	fi
		
	if [ "$6" = "debug" ]; then
		echo "$2 $COUNT $PASSCOUNT $FAILCOUNT"
		echo "Result: $RESULT"
		echo "Result2: $RESULT2"
		echo "------------------------------------" >> $2.debuglog
		echo "$2 $COUNT $PASSCOUNT $FAILCOUNT"  >> $2.debuglog
		echo "Result: $RESULT" >> $2.debuglog
	fi

		
	
    return 0
}

setbaud() {
	if [ -c $1 ]; then
		stty -F $1 $BAUD
	fi
}
case "$1" in
	poll_serial_port)
		serial_response $2 $3 $4 $5 $6 $7
		;;
    micro_monitor) #used when you are looping back the serial port to itself
		echo "Micro Monitor"
		if [ -z "$2" ]; then
		   SERIAL_PORT=/dev/ttyACM0
		else
		   SERIAL_PORT=$2
		fi

		if [ -z "$3" ]; then
		   RESET=""
		else
		   RESET=$3
		fi

		if [ -z "$4" ]; then
		   DEBUG=""
		else
		   DEBUG=$4
		fi
		PRETTY_SERIAL_PORT=${SERIAL_PORT##/*/}
		COUNT_FILE=/var/volatile/${PRETTY_SERIAL_PORT}.count
		FAILCOUNT_FILE=/var/volatile/${PRETTY_SERIAL_PORT}.failcount
		PASSCOUNT_FILE=/var/volatile/${PRETTY_SERIAL_PORT}.passcount
		LOG_FILE=/var/volatile/${PRETTY_SERIAL_PORT}.log

		if [ "$RESET" = "reset" ]; then
			echo 0 > $COUNT_FILE
			echo 0 > $FAILCOUNT_FILE
			echo 0 > $PASSCOUNT_FILE
			echo 0 > $LOG_FILE
		fi
				
        while [ 1 ]; do
			/usr/local/src/selftest/micro_monitor_collection.sh poll_serial_port $SERIAL_PORT $COUNT_FILE $PASSCOUNT_FILE $FAILCOUNT_FILE $LOG_FILE&
			sleep 1
			printf $SERIAL_COMMAND > $SERIAL_PORT
			sleep $TIMEOUT
		done
        ;;
	*)
		echo "Usage: $SCRIPTNAME {micro_monitor} <serial port> <reset counts> <debug>" >&2
	exit 3
	;;
esac



