#!/bin/bash

set -x
#echo "Logging Serial stuff"

_now=$(date +%Y_%m_%d_%H%M%S)        

BAUD="115200"
SERIALMATCHSTRING="Viper4TestStringABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqurstuvwxyz"
TIMEOUT=1

serial_response() {

	if [ ! -c $1 ]; then
		echo "$1 Doesn't Exist"
		return 1
	fi
	
	COUNT=`cat $2`
	PASSCOUNT=`cat $3`
	FAILCOUNT=`cat $4`
	
	RESULT=`perl -e "alarm $TIMEOUT; exec @ARGV" "cat $1"`
	RESULT2=`echo $RESULT | grep "$SERIALMATCHSTRING"`
	COUNT=`expr $COUNT + 1`
	if [ "$RESULT2" ]; then  
		PASSCOUNT=`expr $PASSCOUNT + 1`
	else
		FAILCOUNT=`expr $FAILCOUNT + 1`
	fi
	echo $COUNT > $2
	echo $PASSCOUNT > $3
	echo $FAILCOUNT > $4

	if [ "$5" = "debug" ]; then
		echo "$1 $COUNT $PASSCOUNT $FAILCOUNT"
		echo "Result: $RESULT"
		echo "Result2: $RESULT2"
		echo "------------------------------------" >> $2.debuglog
		echo "$COUNT $PASSCOUNT $FAILCOUNT"  >> $2.debuglog
		echo "Result: $RESULT" >> $2.debuglog
	fi

		
	
    return 0
}

	
setbaud() {
	if [ -c $1 ]; then
		stty -F $1 $BAUD $HANDSHAKING
	fi
}
case "$1" in
	poll_serial_port)
		serial_response $2 $3 $4 $5 $6
		;;
    serial_loopback) #used when you are looping back the serial port to itself
		echo "Serial Loopback"
		if [ -z "$2" ]; then
		   SERIAL_PORT=/dev/ttyO3
		else
		   SERIAL_PORT=$2
		fi

		if [ -z "$3" ]; then
		   BAUD=19200
		else
		   BAUD=$3
		fi

		if [ -z "$4" ]; then
		   RESET=""
		else
		   RESET=$4
		fi

		if [ -z "$5" ]; then
		   DEBUG=""
		else
		   DEBUG=$5
		fi
		
		if [ -z "$6" ]; then
			HANDSHAKING="-crtscts"
		else
			HANDSHAKING="crtscts"
		fi
			
		PRETTY_SERIAL_PORT=${SERIAL_PORT##/*/}
		COUNT_FILE=/var/volatile/${PRETTY_SERIAL_PORT}.count
		FAILCOUNT_FILE=/var/volatile/${PRETTY_SERIAL_PORT}.failcount
		PASSCOUNT_FILE=/var/volatile/${PRETTY_SERIAL_PORT}.passcount

		if [ "$4" = "reset" ]; then
			echo 0 > $COUNT_FILE
			echo 0 > $FAILCOUNT_FILE
			echo 0 > $PASSCOUNT_FILE
		fi
				
        while [ 1 ]; do
			setbaud $SERIAL_PORT #set the baud each time in case it gets disconnected and re-connected?
			/usr/local/src/selftest/seriallooptest.sh poll_serial_port $SERIAL_PORT $COUNT_FILE $PASSCOUNT_FILE $FAILCOUNT_FILE $DEBUG &
			sleep 1
			echo $SERIALMATCHSTRING > $SERIAL_PORT
			sleep $TIMEOUT
		done
        ;;
	serial_bridge) #used when you are looping 2 serial ports together...
		if [ -z "$2" ]; then
		   SERIAL_PORT_A=/dev/ttyUSB0
		else
		   SERIAL_PORT_A=$2
		fi
		if [ -z "$3" ]; then
		   SERIAL_PORT_B=/dev/ttyUSB1
		else
		   SERIAL_PORT_B=$3
		fi
		if [ -z "$4" ]; then
		   BAUD=19200
		else
		   BAUD=$4
		fi

		if [ -z "$5" ]; then
		   RESET=""
		else
		   RESET=$5
		fi

		if [ -z "$6" ]; then
		   DEBUG=""
		else
		   DEBUG=$6
		fi
		
		PRETTY_SERIAL_PORT_A=${SERIAL_PORT_A##/*/}
		PRETTY_SERIAL_PORT_B=${SERIAL_PORT_B##/*/}
		COUNT_FILE_A=/var/volatile/${PRETTY_SERIAL_PORT_A}.count
		COUNT_FILE_B=/var/volatile/${PRETTY_SERIAL_PORT_B}.count
		FAILCOUNT_FILE_A=/var/volatile/${PRETTY_SERIAL_PORT_A}.failcount
		FAILCOUNT_FILE_B=/var/volatile/${PRETTY_SERIAL_PORT_B}.failcount
		PASSCOUNT_FILE_A=/var/volatile/${PRETTY_SERIAL_PORT_A}.passcount
		PASSCOUNT_FILE_B=/var/volatile/${PRETTY_SERIAL_PORT_B}.passcount

		if [ "$RESET" = "reset" ]; then
			echo 0 > $COUNT_FILE_A
			echo 0 > $FAILCOUNT_FILE_A
			echo 0 > $PASSCOUNT_FILE_A
			echo 0 > $COUNT_FILE_B
			echo 0 > $FAILCOUNT_FILE_B
			echo 0 > $PASSCOUNT_FILE_B
		fi

		HANDSHAKING="-crtscts"
		setbaud $SERIAL_PORT_A  
		setbaud $SERIAL_PORT_B 

		sleep $TIMEOUT
		
        while [ 1 ]; do
			#Send out A and listen on B
			/usr/local/src/selftest/seriallooptest.sh poll_serial_port $SERIAL_PORT_B $COUNT_FILE_B $PASSCOUNT_FILE_B $FAILCOUNT_FILE_B $DEBUG &			
			sleep 1
			echo $SERIALMATCHSTRING > $SERIAL_PORT_A
			sleep $TIMEOUT
			#Send out B and listen on A
			/usr/local/src/selftest/seriallooptest.sh poll_serial_port $SERIAL_PORT_A $COUNT_FILE_A $PASSCOUNT_FILE_A $FAILCOUNT_FILE_A $DEBUG &			
			sleep 1
			echo $SERIALMATCHSTRING > $SERIAL_PORT_B
			sleep $TIMEOUT
		done
        ;;
esac



